@@ -41,17 +41,20 @@
 template <typename PointT, typename Scalar> void
 pcl::transformPointCloud (const pcl::PointCloud<PointT> &cloud_in, 
                           pcl::PointCloud<PointT> &cloud_out,
-                          const Eigen::Transform<Scalar, 3, Eigen::Affine> &transform)
+                          const Eigen::Transform<Scalar, 3, Eigen::Affine> &transform,
+                          bool copy_all_fields)
 {
   if (&cloud_in != &cloud_out)
   {
-    // Note: could be replaced by cloud_out = cloud_in
     cloud_out.header   = cloud_in.header;
     cloud_out.is_dense = cloud_in.is_dense;
     cloud_out.width    = cloud_in.width;
     cloud_out.height   = cloud_in.height;
-    cloud_out.points.reserve (cloud_out.points.size ());
-    cloud_out.points.assign (cloud_in.points.begin (), cloud_in.points.end ());
+    cloud_out.points.reserve (cloud_in.points.size ());
+    if (copy_all_fields)
+      cloud_out.points.assign (cloud_in.points.begin (), cloud_in.points.end ());
+    else
+      cloud_out.points.resize (cloud_in.points.size ());
     cloud_out.sensor_orientation_ = cloud_in.sensor_orientation_;
     cloud_out.sensor_origin_      = cloud_in.sensor_origin_;
   }
@@ -92,7 +95,8 @@ template <typename PointT, typename Scalar> void
 pcl::transformPointCloud (const pcl::PointCloud<PointT> &cloud_in, 
                           const std::vector<int> &indices, 
                           pcl::PointCloud<PointT> &cloud_out,
-                          const Eigen::Transform<Scalar, 3, Eigen::Affine> &transform)
+                          const Eigen::Transform<Scalar, 3, Eigen::Affine> &transform,
+                          bool copy_all_fields)
 {
   size_t npts = indices.size ();
   // In order to transform the data, we need to remove NaNs
@@ -110,7 +114,8 @@ pcl::transformPointCloud (const pcl::PointCloud<PointT> &cloud_in,
     for (size_t i = 0; i < npts; ++i)
     {
       // Copy fields first, then transform xyz data
-      //cloud_out.points[i] = cloud_in.points[indices[i]]; 
+      if (copy_all_fields)
+        cloud_out.points[i] = cloud_in.points[indices[i]];
       //cloud_out.points[i].getVector3fMap () = transform*cloud_out.points[i].getVector3fMap ();
       Eigen::Matrix<Scalar, 3, 1> pt (cloud_in[indices[i]].x, cloud_in[indices[i]].y, cloud_in[indices[i]].z);
       cloud_out[i].x = static_cast<float> (transform (0, 0) * pt.coeffRef (0) + transform (0, 1) * pt.coeffRef (1) + transform (0, 2) * pt.coeffRef (2) + transform (0, 3));
@@ -124,11 +129,12 @@ pcl::transformPointCloud (const pcl::PointCloud<PointT> &cloud_in,
     // otherwise we get errors during the multiplication (?)
     for (size_t i = 0; i < npts; ++i)
     {
+      if (copy_all_fields)
+        cloud_out.points[i] = cloud_in.points[indices[i]];
       if (!pcl_isfinite (cloud_in.points[indices[i]].x) || 
           !pcl_isfinite (cloud_in.points[indices[i]].y) || 
           !pcl_isfinite (cloud_in.points[indices[i]].z))
         continue;
-      //cloud_out.points[i] = cloud_in.points[indices[i]]; 
       //cloud_out.points[i].getVector3fMap () = transform*cloud_out.points[i].getVector3fMap ();
       Eigen::Matrix<Scalar, 3, 1> pt (cloud_in[indices[i]].x, cloud_in[indices[i]].y, cloud_in[indices[i]].z);
       cloud_out[i].x = static_cast<float> (transform (0, 0) * pt.coeffRef (0) + transform (0, 1) * pt.coeffRef (1) + transform (0, 2) * pt.coeffRef (2) + transform (0, 3));
@@ -142,7 +148,8 @@ pcl::transformPointCloud (const pcl::PointCloud<PointT> &cloud_in,
 template <typename PointT, typename Scalar> void
 pcl::transformPointCloudWithNormals (const pcl::PointCloud<PointT> &cloud_in, 
                                      pcl::PointCloud<PointT> &cloud_out,
-                                     const Eigen::Transform<Scalar, 3, Eigen::Affine> &transform)
+                                     const Eigen::Transform<Scalar, 3, Eigen::Affine> &transform,
+                                     bool copy_all_fields)
 {
   if (&cloud_in != &cloud_out)
   {
@@ -152,7 +159,10 @@ pcl::transformPointCloudWithNormals (const pcl::PointCloud<PointT> &cloud_in,
     cloud_out.height   = cloud_in.height;
     cloud_out.is_dense = cloud_in.is_dense;
     cloud_out.points.reserve (cloud_out.points.size ());
-    cloud_out.points.assign (cloud_in.points.begin (), cloud_in.points.end ());
+    if (copy_all_fields)
+      cloud_out.points.assign (cloud_in.points.begin (), cloud_in.points.end ());
+    else
+      cloud_out.points.resize (cloud_in.points.size ());
     cloud_out.sensor_orientation_ = cloud_in.sensor_orientation_;
     cloud_out.sensor_origin_      = cloud_in.sensor_origin_;
   }
@@ -207,7 +217,8 @@ template <typename PointT, typename Scalar> void
 pcl::transformPointCloudWithNormals (const pcl::PointCloud<PointT> &cloud_in, 
                                      const std::vector<int> &indices, 
                                      pcl::PointCloud<PointT> &cloud_out,
-                                     const Eigen::Transform<Scalar, 3, Eigen::Affine> &transform)
+                                     const Eigen::Transform<Scalar, 3, Eigen::Affine> &transform,
+                                     bool copy_all_fields)
 {
   size_t npts = indices.size ();
   // In order to transform the data, we need to remove NaNs
@@ -224,6 +235,9 @@ pcl::transformPointCloudWithNormals (const pcl::PointCloud<PointT> &cloud_in,
   {
     for (size_t i = 0; i < cloud_out.points.size (); ++i)
     {
+      // Copy fields first, then transform
+      if (copy_all_fields)
+        cloud_out.points[i] = cloud_in.points[indices[i]];
       //cloud_out.points[i].getVector3fMap() = transform * cloud_in.points[i].getVector3fMap ();
       Eigen::Matrix<Scalar, 3, 1> pt (cloud_in[indices[i]].x, cloud_in[indices[i]].y, cloud_in[indices[i]].z);
       cloud_out[i].x = static_cast<float> (transform (0, 0) * pt.coeffRef (0) + transform (0, 1) * pt.coeffRef (1) + transform (0, 2) * pt.coeffRef (2) + transform (0, 3));
@@ -243,6 +257,10 @@ pcl::transformPointCloudWithNormals (const pcl::PointCloud<PointT> &cloud_in,
   {
     for (size_t i = 0; i < cloud_out.points.size (); ++i)
     {
+      // Copy fields first, then transform
+      if (copy_all_fields)
+        cloud_out.points[i] = cloud_in.points[indices[i]];
+
       if (!pcl_isfinite (cloud_in.points[indices[i]].x) || 
           !pcl_isfinite (cloud_in.points[indices[i]].y) || 
           !pcl_isfinite (cloud_in.points[indices[i]].z))
@@ -269,12 +287,13 @@ template <typename PointT, typename Scalar> inline void
 pcl::transformPointCloud (const pcl::PointCloud<PointT> &cloud_in, 
                           pcl::PointCloud<PointT> &cloud_out,
                           const Eigen::Matrix<Scalar, 3, 1> &offset, 
-                          const Eigen::Quaternion<Scalar> &rotation)
+                          const Eigen::Quaternion<Scalar> &rotation,
+                          bool copy_all_fields)
 {
   Eigen::Translation<Scalar, 3> translation (offset);
   // Assemble an Eigen Transform
   Eigen::Transform<Scalar, 3, Eigen::Affine> t (translation * rotation);
-  transformPointCloud (cloud_in, cloud_out, t);
+  transformPointCloud (cloud_in, cloud_out, t, copy_all_fields);
 }
 
 ///////////////////////////////////////////////////////////////////////////////////////////
@@ -282,12 +301,13 @@ template <typename PointT, typename Scalar> inline void
 pcl::transformPointCloudWithNormals (const pcl::PointCloud<PointT> &cloud_in, 
                                      pcl::PointCloud<PointT> &cloud_out,
                                      const Eigen::Matrix<Scalar, 3, 1> &offset, 
-                                     const Eigen::Quaternion<Scalar> &rotation)
+                                     const Eigen::Quaternion<Scalar> &rotation,
+                                     bool copy_all_fields)
 {
   Eigen::Translation<Scalar, 3> translation (offset);
   // Assemble an Eigen Transform
   Eigen::Transform<Scalar, 3, Eigen::Affine> t (translation * rotation);
-  transformPointCloudWithNormals (cloud_in, cloud_out, t);
+  transformPointCloudWithNormals (cloud_in, cloud_out, t, copy_all_fields);
 }
 
 ///////////////////////////////////////////////////////////////////////////////////////////
@@ -296,10 +316,19 @@ pcl::transformPoint (const PointT &point,
                      const Eigen::Transform<Scalar, 3, Eigen::Affine> &transform)
 {
   PointT ret = point;
-  //ret.getVector3fMap () = transform * point.getVector3fMap ();
-  ret.x = static_cast<float> (transform (0, 0) * point.x + transform (0, 1) * point.y + transform (0, 2) * point.z + transform (0, 3));
-  ret.y = static_cast<float> (transform (1, 0) * point.x + transform (1, 1) * point.y + transform (1, 2) * point.z + transform (1, 3));
-  ret.z = static_cast<float> (transform (2, 0) * point.x + transform (2, 1) * point.y + transform (2, 2) * point.z + transform (2, 3));
+  ret.getVector3fMap () = transform * point.getVector3fMap ();
+
+  return (ret);
+}
+
+///////////////////////////////////////////////////////////////////////////////////////////
+template <typename PointT, typename Scalar> inline PointT
+pcl::transformPointWithNormal (const PointT &point, 
+                     const Eigen::Transform<Scalar, 3, Eigen::Affine> &transform)
+{
+  PointT ret = point;
+  ret.getVector3fMap () = transform * point.getVector3fMap ();
+  ret.getNormalVector3fMap () = transform.rotation () * point.getNormalVector3fMap ();
 
   return (ret);
 }
