@@ -2,7 +2,8 @@
  * Software License Agreement (BSD License)
  *
  *  Point Cloud Library (PCL) - www.pointclouds.org
- *  Copyright (c) 2012 The MITRE Corporation
+ *  Copyright (c) 2012-, Open Perception, Inc.
+ *  Copyright (c) 2012,2015 The MITRE Corporation
  *
  *  All rights reserved.
  *
@@ -49,7 +50,6 @@
 #include <pcap.h>
 #endif // #ifdef HAVE_PCAP
 
-const boost::asio::ip::address pcl::HDLGrabber::HDL_DEFAULT_NETWORK_ADDRESS = boost::asio::ip::address::from_string ("192.168.3.255");
 double *pcl::HDLGrabber::cos_lookup_table_ = NULL;
 double *pcl::HDLGrabber::sin_lookup_table_ = NULL;
 
@@ -57,63 +57,63 @@ using boost::asio::ip::udp;
 
 /////////////////////////////////////////////////////////////////////////////
 pcl::HDLGrabber::HDLGrabber (const std::string& correctionsFile,
-                             const std::string& pcapFile) 
-  : hdl_data_ ()
-  , udp_listener_endpoint_ (HDL_DEFAULT_NETWORK_ADDRESS, HDL_DATA_PORT)
-  , source_address_filter_ ()
-  , source_port_filter_ (443)
-  , hdl_read_socket_service_ ()
-  , hdl_read_socket_ (NULL)
-  , pcap_file_name_ (pcapFile)
-  , queue_consumer_thread_ (NULL)
-  , hdl_read_packet_thread_ (NULL)
-  , current_scan_xyz_ (new pcl::PointCloud<pcl::PointXYZ> ())
-  , current_sweep_xyz_ (new pcl::PointCloud<pcl::PointXYZ> ())
-  , current_scan_xyzi_ (new pcl::PointCloud<pcl::PointXYZI> ())
-  , current_sweep_xyzi_ (new pcl::PointCloud<pcl::PointXYZI> ())
-  , current_scan_xyzrgb_ (new pcl::PointCloud<pcl::PointXYZRGBA> ())
-  , current_sweep_xyzrgb_ (new pcl::PointCloud<pcl::PointXYZRGBA> ())
-  , last_azimuth_ (65000)
-  , sweep_xyz_signal_ ()
-  , sweep_xyzrgb_signal_ ()
-  , sweep_xyzi_signal_ ()
-  , scan_xyz_signal_ ()
-  , scan_xyzrgb_signal_ ()
-  , scan_xyzi_signal_ ()
-  , min_distance_threshold_(0.0)
-  , max_distance_threshold_(10000.0)
+                             const std::string& pcapFile) :
+    last_azimuth_ (65000),
+    current_scan_xyz_ (new pcl::PointCloud<pcl::PointXYZ> ()),
+    current_sweep_xyz_ (new pcl::PointCloud<pcl::PointXYZ> ()),
+    current_scan_xyzi_ (new pcl::PointCloud<pcl::PointXYZI> ()),
+    current_sweep_xyzi_ (new pcl::PointCloud<pcl::PointXYZI> ()),
+    current_scan_xyzrgb_ (new pcl::PointCloud<pcl::PointXYZRGBA> ()),
+    current_sweep_xyzrgb_ (new pcl::PointCloud<pcl::PointXYZRGBA> ()),
+    sweep_xyz_signal_ (),
+    sweep_xyzrgb_signal_ (),
+    sweep_xyzi_signal_ (),
+    scan_xyz_signal_ (),
+    scan_xyzrgb_signal_ (),
+    scan_xyzi_signal_ (),
+    hdl_data_ (),
+    udp_listener_endpoint_ (),
+    source_address_filter_ (),
+    source_port_filter_ (443),
+    hdl_read_socket_service_ (),
+    hdl_read_socket_ (NULL),
+    pcap_file_name_ (pcapFile),
+    queue_consumer_thread_ (NULL),
+    hdl_read_packet_thread_ (NULL),
+    min_distance_threshold_ (0.0),
+    max_distance_threshold_ (10000.0)
 {
   initialize (correctionsFile);
 }
 
 /////////////////////////////////////////////////////////////////////////////
 pcl::HDLGrabber::HDLGrabber (const boost::asio::ip::address& ipAddress,
-                             const unsigned short int port, 
-                             const std::string& correctionsFile) 
-  : hdl_data_ ()
-  , udp_listener_endpoint_ (ipAddress, port)
-  , source_address_filter_ ()
-  , source_port_filter_ (443)
-  , hdl_read_socket_service_ ()
-  , hdl_read_socket_ (NULL)
-  , pcap_file_name_ ()
-  , queue_consumer_thread_ (NULL)
-  , hdl_read_packet_thread_ (NULL)
-  , current_scan_xyz_ (new pcl::PointCloud<pcl::PointXYZ> ())
-  , current_sweep_xyz_ (new pcl::PointCloud<pcl::PointXYZ> ())
-  , current_scan_xyzi_ (new pcl::PointCloud<pcl::PointXYZI> ())
-  , current_sweep_xyzi_ (new pcl::PointCloud<pcl::PointXYZI> ())
-  , current_scan_xyzrgb_ (new pcl::PointCloud<pcl::PointXYZRGBA> ())
-  , current_sweep_xyzrgb_ (new pcl::PointCloud<pcl::PointXYZRGBA> ())
-  , last_azimuth_ (65000)
-  , sweep_xyz_signal_ ()
-  , sweep_xyzrgb_signal_ ()
-  , sweep_xyzi_signal_ ()
-  , scan_xyz_signal_ ()
-  , scan_xyzrgb_signal_ ()
-  , scan_xyzi_signal_ ()
-  , min_distance_threshold_(0.0)
-  , max_distance_threshold_(10000.0)
+                             const unsigned short int port,
+                             const std::string& correctionsFile) :
+    last_azimuth_ (65000),
+    current_scan_xyz_ (new pcl::PointCloud<pcl::PointXYZ> ()),
+    current_sweep_xyz_ (new pcl::PointCloud<pcl::PointXYZ> ()),
+    current_scan_xyzi_ (new pcl::PointCloud<pcl::PointXYZI> ()),
+    current_sweep_xyzi_ (new pcl::PointCloud<pcl::PointXYZI> ()),
+    current_scan_xyzrgb_ (new pcl::PointCloud<pcl::PointXYZRGBA> ()),
+    current_sweep_xyzrgb_ (new pcl::PointCloud<pcl::PointXYZRGBA> ()),
+    sweep_xyz_signal_ (),
+    sweep_xyzrgb_signal_ (),
+    sweep_xyzi_signal_ (),
+    scan_xyz_signal_ (),
+    scan_xyzrgb_signal_ (),
+    scan_xyzi_signal_ (),
+    hdl_data_ (),
+    udp_listener_endpoint_ (ipAddress, port),
+    source_address_filter_ (),
+    source_port_filter_ (443),
+    hdl_read_socket_service_ (),
+    hdl_read_socket_ (NULL),
+    pcap_file_name_ (),
+    queue_consumer_thread_ (NULL),
+    hdl_read_packet_thread_ (NULL),
+    min_distance_threshold_ (0.0),
+    max_distance_threshold_ (10000.0)
 {
   initialize (correctionsFile);
 }
@@ -139,7 +139,7 @@ pcl::HDLGrabber::initialize (const std::string& correctionsFile)
   {
     cos_lookup_table_ = static_cast<double *> (malloc (HDL_NUM_ROT_ANGLES * sizeof (*cos_lookup_table_)));
     sin_lookup_table_ = static_cast<double *> (malloc (HDL_NUM_ROT_ANGLES * sizeof (*sin_lookup_table_)));
-    for (unsigned int i = 0; i < HDL_NUM_ROT_ANGLES; i++)
+    for (int i = 0; i < HDL_NUM_ROT_ANGLES; i++)
     {
       double rad = (M_PI / 180.0) * (static_cast<double> (i) / 100.0);
       cos_lookup_table_[i] = std::cos (rad);
@@ -152,14 +152,12 @@ pcl::HDLGrabber::initialize (const std::string& correctionsFile)
   for (int i = 0; i < HDL_MAX_NUM_LASERS; i++)
   {
     HDLLaserCorrection correction = laser_corrections_[i];
-    laser_corrections_[i].sinVertOffsetCorrection = correction.verticalOffsetCorrection
-                                       * correction.sinVertCorrection;
-    laser_corrections_[i].cosVertOffsetCorrection = correction.verticalOffsetCorrection
-                                       * correction.cosVertCorrection;
+    laser_corrections_[i].sinVertOffsetCorrection = correction.verticalOffsetCorrection * correction.sinVertCorrection;
+    laser_corrections_[i].cosVertOffsetCorrection = correction.verticalOffsetCorrection * correction.cosVertCorrection;
   }
   sweep_xyz_signal_ = createSignal<sig_cb_velodyne_hdl_sweep_point_cloud_xyz> ();
   sweep_xyzrgb_signal_ = createSignal<sig_cb_velodyne_hdl_sweep_point_cloud_xyzrgb> ();
-  sweep_xyzi_signal_ =createSignal<sig_cb_velodyne_hdl_sweep_point_cloud_xyzi> ();
+  sweep_xyzi_signal_ = createSignal<sig_cb_velodyne_hdl_sweep_point_cloud_xyzi> ();
   scan_xyz_signal_ = createSignal<sig_cb_velodyne_hdl_scan_point_cloud_xyz> ();
   scan_xyzrgb_signal_ = createSignal<sig_cb_velodyne_hdl_scan_point_cloud_xyzrgb> ();
   scan_xyzi_signal_ = createSignal<sig_cb_velodyne_hdl_scan_point_cloud_xyzi> ();
@@ -225,33 +223,32 @@ pcl::HDLGrabber::loadCorrectionsFile (const std::string& correctionsFile)
       {
         if (px.first == "px")
         {
-          boost::property_tree::ptree calibrationData = px.second;
+          boost::property_tree::ptree calibration_data = px.second;
           int index = -1;
-          double azimuth = 0, vertCorrection = 0, distCorrection = 0,
-                 vertOffsetCorrection = 0, horizOffsetCorrection = 0;
+          double azimuth = 0, vert_correction = 0, dist_correction = 0, vert_offset_correction = 0, horiz_offset_correction = 0;
 
-          BOOST_FOREACH (boost::property_tree::ptree::value_type &item, calibrationData)
+          BOOST_FOREACH (boost::property_tree::ptree::value_type &item, calibration_data)
           {
             if (item.first == "id_")
               index = atoi (item.second.data ().c_str ());
             if (item.first == "rotCorrection_")
               azimuth = atof (item.second.data ().c_str ());
             if (item.first == "vertCorrection_")
-              vertCorrection = atof (item.second.data ().c_str ());
+              vert_correction = atof (item.second.data ().c_str ());
             if (item.first == "distCorrection_")
-              distCorrection = atof (item.second.data ().c_str ());
+              dist_correction = atof (item.second.data ().c_str ());
             if (item.first == "vertOffsetCorrection_")
-              vertOffsetCorrection = atof (item.second.data ().c_str ());
+              vert_offset_correction = atof (item.second.data ().c_str ());
             if (item.first == "horizOffsetCorrection_")
-              horizOffsetCorrection = atof (item.second.data ().c_str ());
+              horiz_offset_correction = atof (item.second.data ().c_str ());
           }
           if (index != -1)
           {
             laser_corrections_[index].azimuthCorrection = azimuth;
-            laser_corrections_[index].verticalCorrection = vertCorrection;
-            laser_corrections_[index].distanceCorrection = distCorrection / 100.0;
-            laser_corrections_[index].verticalOffsetCorrection = vertOffsetCorrection / 100.0;
-            laser_corrections_[index].horizontalOffsetCorrection = horizOffsetCorrection / 100.0;
+            laser_corrections_[index].verticalCorrection = vert_correction;
+            laser_corrections_[index].distanceCorrection = dist_correction / 100.0;
+            laser_corrections_[index].verticalOffsetCorrection = vert_offset_correction / 100.0;
+            laser_corrections_[index].horizontalOffsetCorrection = horiz_offset_correction / 100.0;
 
             laser_corrections_[index].cosVertCorrection = std::cos (HDL_Grabber_toRadians(laser_corrections_[index].verticalCorrection));
             laser_corrections_[index].sinVertCorrection = std::sin (HDL_Grabber_toRadians(laser_corrections_[index].verticalCorrection));
@@ -266,20 +263,17 @@ pcl::HDLGrabber::loadCorrectionsFile (const std::string& correctionsFile)
 void
 pcl::HDLGrabber::loadHDL32Corrections ()
 {
-  double hdl32VerticalCorrections[] = { 
-    -30.67, -9.3299999, -29.33, -8, -28,
-    -6.6700001, -26.67, -5.3299999, -25.33, -4, -24, -2.6700001, -22.67,
-    -1.33, -21.33, 0, -20, 1.33, -18.67, 2.6700001, -17.33, 4, -16, 5.3299999,
-    -14.67, 6.6700001, -13.33, 8, -12, 9.3299999, -10.67, 10.67 };
+  double hdl32_vertical_corrections[] = { -30.67, -9.3299999, -29.33, -8, -28, -6.6700001, -26.67, -5.3299999, -25.33, -4, -24, -2.6700001, -22.67, -1.33, -21.33,
+      0, -20, 1.33, -18.67, 2.6700001, -17.33, 4, -16, 5.3299999, -14.67, 6.6700001, -13.33, 8, -12, 9.3299999, -10.67, 10.67 };
   for (int i = 0; i < HDL_LASER_PER_FIRING; i++)
   {
     laser_corrections_[i].azimuthCorrection = 0.0;
     laser_corrections_[i].distanceCorrection = 0.0;
     laser_corrections_[i].horizontalOffsetCorrection = 0.0;
     laser_corrections_[i].verticalOffsetCorrection = 0.0;
-    laser_corrections_[i].verticalCorrection = hdl32VerticalCorrections[i];
-    laser_corrections_[i].sinVertCorrection = std::sin (HDL_Grabber_toRadians(hdl32VerticalCorrections[i]));
-    laser_corrections_[i].cosVertCorrection = std::cos (HDL_Grabber_toRadians(hdl32VerticalCorrections[i]));
+    laser_corrections_[i].verticalCorrection = hdl32_vertical_corrections[i];
+    laser_corrections_[i].sinVertCorrection = std::sin (HDL_Grabber_toRadians(hdl32_vertical_corrections[i]));
+    laser_corrections_[i].cosVertCorrection = std::cos (HDL_Grabber_toRadians(hdl32_vertical_corrections[i]));
   }
   for (int i = HDL_LASER_PER_FIRING; i < HDL_MAX_NUM_LASERS; i++)
   {
@@ -293,6 +287,13 @@ pcl::HDLGrabber::loadHDL32Corrections ()
   }
 }
 
+/////////////////////////////////////////////////////////////////////////////
+boost::asio::ip::address
+pcl::HDLGrabber::getDefaultNetworkAddress ()
+{
+  return (boost::asio::ip::address::from_string ("192.168.3.255"));
+}
+
 /////////////////////////////////////////////////////////////////////////////
 void
 pcl::HDLGrabber::processVelodynePackets ()
@@ -313,47 +314,47 @@ pcl::HDLGrabber::processVelodynePackets ()
 void
 pcl::HDLGrabber::toPointClouds (HDLDataPacket *dataPacket)
 {
-  static uint32_t scanCounter = 0;
-  static uint32_t sweepCounter = 0;
-  if (sizeof (HDLLaserReturn) != 3)
+  static uint32_t scan_counter = 0;
+  static uint32_t sweep_counter = 0;
+  if (sizeof(HDLLaserReturn) != 3)
     return;
 
   current_scan_xyz_.reset (new pcl::PointCloud<pcl::PointXYZ> ());
   current_scan_xyzrgb_.reset (new pcl::PointCloud<pcl::PointXYZRGBA> ());
   current_scan_xyzi_.reset (new pcl::PointCloud<pcl::PointXYZI> ());
 
-  time_t  time_;
-  time(&time_);
-  time_t velodyneTime = (time_ & 0x00000000ffffffffl) << 32 | dataPacket->gpsTimestamp;
+  time_t system_time;
+  time (&system_time);
+  time_t velodyne_time = (system_time & 0x00000000ffffffffl) << 32 | dataPacket->gpsTimestamp;
 
-  current_scan_xyz_->header.stamp = velodyneTime;
-  current_scan_xyzrgb_->header.stamp = velodyneTime;
-  current_scan_xyzi_->header.stamp = velodyneTime;
-  current_scan_xyz_->header.seq = scanCounter;
-  current_scan_xyzrgb_->header.seq = scanCounter;
-  current_scan_xyzi_->header.seq = scanCounter;
-  scanCounter++;
+  current_scan_xyz_->header.stamp = velodyne_time;
+  current_scan_xyzrgb_->header.stamp = velodyne_time;
+  current_scan_xyzi_->header.stamp = velodyne_time;
+  current_scan_xyz_->header.seq = scan_counter;
+  current_scan_xyzrgb_->header.seq = scan_counter;
+  current_scan_xyzi_->header.seq = scan_counter;
+  scan_counter++;
 
   for (int i = 0; i < HDL_FIRING_PER_PKT; ++i)
   {
-    HDLFiringData firingData = dataPacket->firingData[i];
-    int offset = (firingData.blockIdentifier == BLOCK_0_TO_31) ? 0 : 32;
+    HDLFiringData firing_data = dataPacket->firingData[i];
+    int offset = (firing_data.blockIdentifier == BLOCK_0_TO_31) ? 0 : 32;
 
     for (int j = 0; j < HDL_LASER_PER_FIRING; j++)
     {
-      if (firingData.rotationalPosition < last_azimuth_)
+      if (firing_data.rotationalPosition < last_azimuth_)
       {
         if (current_sweep_xyzrgb_->size () > 0)
         {
           current_sweep_xyz_->is_dense = current_sweep_xyzrgb_->is_dense = current_sweep_xyzi_->is_dense = false;
-          current_sweep_xyz_->header.stamp = velodyneTime;
-          current_sweep_xyzrgb_->header.stamp = velodyneTime;
-          current_sweep_xyzi_->header.stamp = velodyneTime;
-          current_sweep_xyz_->header.seq = sweepCounter;
-          current_sweep_xyzrgb_->header.seq = sweepCounter;
-          current_sweep_xyzi_->header.seq = sweepCounter;
+          current_sweep_xyz_->header.stamp = velodyne_time;
+          current_sweep_xyzrgb_->header.stamp = velodyne_time;
+          current_sweep_xyzi_->header.stamp = velodyne_time;
+          current_sweep_xyz_->header.seq = sweep_counter;
+          current_sweep_xyzrgb_->header.seq = sweep_counter;
+          current_sweep_xyzi_->header.seq = sweep_counter;
 
-          sweepCounter++;
+          sweep_counter++;
 
           fireCurrentSweep ();
         }
@@ -366,16 +367,15 @@ pcl::HDLGrabber::toPointClouds (HDLDataPacket *dataPacket)
       PointXYZI xyzi;
       PointXYZRGBA xyzrgb;
 
-      computeXYZI (xyzi, firingData.rotationalPosition, firingData.laserReturns[j], laser_corrections_[j + offset]);
+      computeXYZI (xyzi, firing_data.rotationalPosition, firing_data.laserReturns[j], laser_corrections_[j + offset]);
 
       xyz.x = xyzrgb.x = xyzi.x;
       xyz.y = xyzrgb.y = xyzi.y;
       xyz.z = xyzrgb.z = xyzi.z;
 
       xyzrgb.rgba = laser_rgb_mapping_[j + offset].rgba;
-      if ((boost::math::isnan)(xyz.x) ||
-          (boost::math::isnan)(xyz.y) ||
-          (boost::math::isnan)(xyz.z)) {
+      if (pcl_isnan (xyz.x) || pcl_isnan (xyz.y) || pcl_isnan (xyz.z))
+      {
         continue;
       }
 
@@ -387,70 +387,75 @@ pcl::HDLGrabber::toPointClouds (HDLDataPacket *dataPacket)
       current_sweep_xyzi_->push_back (xyzi);
       current_sweep_xyzrgb_->push_back (xyzrgb);
 
-      last_azimuth_ = firingData.rotationalPosition;
+      last_azimuth_ = firing_data.rotationalPosition;
     }
   }
 
   current_scan_xyz_->is_dense = current_scan_xyzrgb_->is_dense = current_scan_xyzi_->is_dense = true;
-  fireCurrentScan (dataPacket->firingData[0].rotationalPosition, 
-                   dataPacket->firingData[11].rotationalPosition);
+  fireCurrentScan (dataPacket->firingData[0].rotationalPosition, dataPacket->firingData[11].rotationalPosition);
 }
 
 /////////////////////////////////////////////////////////////////////////////
 void
-pcl::HDLGrabber::computeXYZI (pcl::PointXYZI& point, int azimuth, 
-                              HDLLaserReturn laserReturn, HDLLaserCorrection correction)
+pcl::HDLGrabber::computeXYZI (pcl::PointXYZI& point,
+                              int azimuth,
+                              HDLLaserReturn laserReturn,
+                              HDLLaserCorrection correction)
 {
-  double cosAzimuth, sinAzimuth;
+  double cos_azimuth, sin_azimuth;
 
   double distanceM = laserReturn.distance * 0.002;
 
-  if (distanceM < min_distance_threshold_ || distanceM > max_distance_threshold_) {
-    point.x = point.y = point.z = std::numeric_limits<float>::quiet_NaN();
-    point.intensity = static_cast<float> (laserReturn.intensity);
+  point.intensity = static_cast<float> (laserReturn.intensity);
+  if (distanceM < min_distance_threshold_ || distanceM > max_distance_threshold_)
+  {
+    point.x = point.y = point.z = std::numeric_limits<float>::quiet_NaN ();
     return;
   }
 
   if (correction.azimuthCorrection == 0)
   {
-    cosAzimuth = cos_lookup_table_[azimuth];
-    sinAzimuth = sin_lookup_table_[azimuth];
+    cos_azimuth = cos_lookup_table_[azimuth];
+    sin_azimuth = sin_lookup_table_[azimuth];
   }
   else
   {
-    double azimuthInRadians = HDL_Grabber_toRadians ((static_cast<double> (azimuth) / 100.0) - correction.azimuthCorrection);
-    cosAzimuth = std::cos (azimuthInRadians);
-    sinAzimuth = std::sin (azimuthInRadians);
+    double azimuthInRadians = HDL_Grabber_toRadians( (static_cast<double> (azimuth) / 100.0) - correction.azimuthCorrection);
+    cos_azimuth = std::cos (azimuthInRadians);
+    sin_azimuth = std::sin (azimuthInRadians);
   }
 
   distanceM += correction.distanceCorrection;
 
-  double xyDistance = distanceM * correction.cosVertCorrection - correction.sinVertOffsetCorrection;
+  double xyDistance = distanceM * correction.cosVertCorrection;
 
-  point.x = static_cast<float> (xyDistance * sinAzimuth - correction.horizontalOffsetCorrection * cosAzimuth);
-  point.y = static_cast<float> (xyDistance * cosAzimuth + correction.horizontalOffsetCorrection * sinAzimuth);
-  point.z = static_cast<float> (distanceM * correction.sinVertCorrection + correction.cosVertOffsetCorrection);
-  point.intensity = static_cast<float> (laserReturn.intensity);
+  point.x = static_cast<float> (xyDistance * sin_azimuth - correction.horizontalOffsetCorrection * cos_azimuth);
+  point.y = static_cast<float> (xyDistance * cos_azimuth + correction.horizontalOffsetCorrection * sin_azimuth);
+  point.z = static_cast<float> (distanceM * correction.sinVertCorrection + correction.verticalOffsetCorrection);
+  if (point.x == 0 && point.y == 0 && point.z == 0)
+  {
+    point.x = point.y = point.z = std::numeric_limits<float>::quiet_NaN ();
+  }
 }
 
 /////////////////////////////////////////////////////////////////////////////
 void
 pcl::HDLGrabber::fireCurrentSweep ()
 {
-  if (sweep_xyz_signal_->num_slots () > 0)
+  if (sweep_xyz_signal_ != NULL && sweep_xyz_signal_->num_slots () > 0)
     sweep_xyz_signal_->operator() (current_sweep_xyz_);
-  
-  if (sweep_xyzrgb_signal_->num_slots () > 0)
+
+  if (sweep_xyzrgb_signal_ != NULL && sweep_xyzrgb_signal_->num_slots () > 0)
     sweep_xyzrgb_signal_->operator() (current_sweep_xyzrgb_);
 
-  if (sweep_xyzi_signal_->num_slots () > 0)
+  if (sweep_xyzi_signal_ != NULL && sweep_xyzi_signal_->num_slots () > 0)
     sweep_xyzi_signal_->operator() (current_sweep_xyzi_);
 }
 
 /////////////////////////////////////////////////////////////////////////////
 void
 pcl::HDLGrabber::fireCurrentScan (const unsigned short startAngle,
-    const unsigned short endAngle)
+                                  const unsigned short endAngle)
 {
   const float start = static_cast<float> (startAngle) / 100.0f;
   const float end = static_cast<float> (endAngle) / 100.0f;
@@ -468,7 +473,7 @@ pcl::HDLGrabber::fireCurrentScan (const unsigned short startAngle,
 /////////////////////////////////////////////////////////////////////////////
 void
 pcl::HDLGrabber::enqueueHDLPacket (const unsigned char *data,
-    std::size_t bytesReceived)
+                                   std::size_t bytesReceived)
 {
   if (bytesReceived == 1206)
   {
@@ -494,19 +499,29 @@ pcl::HDLGrabber::start ()
   {
     try
     {
-      try {
-		  hdl_read_socket_ = new udp::socket (hdl_read_socket_service_, udp_listener_endpoint_);
-	  }
-	  catch (std::exception bind) {
-		  delete hdl_read_socket_;
-		  hdl_read_socket_ = new udp::socket (hdl_read_socket_service_, udp::endpoint(boost::asio::ip::address_v4::any(), udp_listener_endpoint_.port()));
-	  }
+      try
+      {
+        if (isAddressUnspecified (udp_listener_endpoint_.address ()))
+        {
+          udp_listener_endpoint_.address (getDefaultNetworkAddress ());
+        }
+        if (udp_listener_endpoint_.port () == 0)
+        {
+          udp_listener_endpoint_.port (HDL_DATA_PORT);
+        }
+        hdl_read_socket_ = new udp::socket (hdl_read_socket_service_, udp_listener_endpoint_);
+      }
+      catch (const std::exception& bind)
+      {
+        delete hdl_read_socket_;
+        hdl_read_socket_ = new udp::socket (hdl_read_socket_service_, udp::endpoint (boost::asio::ip::address_v4::any (), udp_listener_endpoint_.port ()));
+      }
       hdl_read_socket_service_.run ();
     }
     catch (std::exception &e)
     {
-		PCL_ERROR ("[pcl::HDLGrabber::start] Unable to bind to socket! %s\n", e.what());
-        return;
+      PCL_ERROR("[pcl::HDLGrabber::start] Unable to bind to socket! %s\n", e.what ());
+      return;
     }
     hdl_read_packet_thread_ = new boost::thread (boost::bind (&HDLGrabber::readPacketsFromSocket, this));
   }
@@ -550,8 +565,7 @@ pcl::HDLGrabber::stop ()
 bool
 pcl::HDLGrabber::isRunning () const
 {
-	return (!hdl_data_.isEmpty() || (hdl_read_packet_thread_ != NULL && 
-         !hdl_read_packet_thread_->timed_join (boost::posix_time::milliseconds (10))));
+  return (!hdl_data_.isEmpty () || (hdl_read_packet_thread_ != NULL && !hdl_read_packet_thread_->timed_join (boost::posix_time::milliseconds (10))));
 }
 
 /////////////////////////////////////////////////////////////////////////////
@@ -582,7 +596,7 @@ void
 pcl::HDLGrabber::setLaserColorRGB (const pcl::RGB& color,
                                    unsigned int laserNumber)
 {
-  if (laserNumber >= HDL_MAX_NUM_LASERS)
+  if (laserNumber >= (unsigned int) HDL_MAX_NUM_LASERS)
     return;
 
   laser_rgb_mapping_[laserNumber] = color;
@@ -604,26 +618,30 @@ pcl::HDLGrabber::isAddressUnspecified (const boost::asio::ip::address& ipAddress
 
 /////////////////////////////////////////////////////////////////////////////
 void
-pcl::HDLGrabber::setMaximumDistanceThreshold(float &maxThreshold) {
+pcl::HDLGrabber::setMaximumDistanceThreshold (float &maxThreshold)
+{
   max_distance_threshold_ = maxThreshold;
 }
 
 /////////////////////////////////////////////////////////////////////////////
 void
-pcl::HDLGrabber::setMinimumDistanceThreshold(float &minThreshold) {
+pcl::HDLGrabber::setMinimumDistanceThreshold (float &minThreshold)
+{
   min_distance_threshold_ = minThreshold;
 }
 
 /////////////////////////////////////////////////////////////////////////////
 float
-pcl::HDLGrabber::getMaximumDistanceThreshold() {
-  return(max_distance_threshold_);
+pcl::HDLGrabber::getMaximumDistanceThreshold ()
+{
+  return (max_distance_threshold_);
 }
 
 /////////////////////////////////////////////////////////////////////////////
 float
-pcl::HDLGrabber::getMinimumDistanceThreshold() {
-  return(min_distance_threshold_);
+pcl::HDLGrabber::getMinimumDistanceThreshold ()
+{
+  return (min_distance_threshold_);
 }
 
 /////////////////////////////////////////////////////////////////////////////
@@ -633,15 +651,15 @@ pcl::HDLGrabber::readPacketsFromSocket ()
   unsigned char data[1500];
   udp::endpoint sender_endpoint;
 
-  while (!terminate_read_packet_thread_ && hdl_read_socket_->is_open())
+  while (!terminate_read_packet_thread_ && hdl_read_socket_->is_open ())
   {
     size_t length = hdl_read_socket_->receive_from (boost::asio::buffer (data, 1500), sender_endpoint);
 
-    if (isAddressUnspecified (source_address_filter_) || 
-        (source_address_filter_ == sender_endpoint.address () && source_port_filter_ == sender_endpoint.port ()))
-	{
+    if (isAddressUnspecified (source_address_filter_)
+        || (source_address_filter_ == sender_endpoint.address () && source_port_filter_ == sender_endpoint.port ()))
+    {
       enqueueHDLPacket (data, length);
-	}
+    }
   }
 }
 
@@ -657,16 +675,16 @@ pcl::HDLGrabber::readPacketsFromPcap ()
   pcap_t *pcap = pcap_open_offline (pcap_file_name_.c_str (), errbuff);
 
   struct bpf_program filter;
-  std::ostringstream stringStream;
+  std::ostringstream string_stream;
 
-  stringStream << "udp ";
+  string_stream << "udp ";
   if (!isAddressUnspecified(source_address_filter_))
   {
-    stringStream << " and src port " << source_port_filter_ << " and src host " << source_address_filter_.to_string();
+    string_stream << " and src port " << source_port_filter_ << " and src host " << source_address_filter_.to_string();
   }
 
   // PCAP_NETMASK_UNKNOWN should be 0xffffffff, but it's undefined in older PCAP versions
-  if (pcap_compile (pcap, &filter, stringStream.str ().c_str(), 0, 0xffffffff) == -1)
+  if (pcap_compile (pcap, &filter, string_stream.str ().c_str(), 0, 0xffffffff) == -1)
   {
     PCL_WARN ("[pcl::HDLGrabber::readPacketsFromPcap] Issue compiling filter: %s.\n", pcap_geterr (pcap));
   }
@@ -676,7 +694,7 @@ pcl::HDLGrabber::readPacketsFromPcap ()
   }
 
   struct timeval lasttime;
-  unsigned long long uSecDelay;
+  unsigned long long usec_delay;
 
   lasttime.tv_sec = 0;
 
@@ -694,10 +712,10 @@ pcl::HDLGrabber::readPacketsFromPcap ()
       lasttime.tv_usec -= 1000000;
       lasttime.tv_sec++;
     }
-    uSecDelay = ((header->ts.tv_sec - lasttime.tv_sec) * 1000000) +
-                (header->ts.tv_usec - lasttime.tv_usec);
+    usec_delay = ((header->ts.tv_sec - lasttime.tv_sec) * 1000000) +
+    (header->ts.tv_usec - lasttime.tv_usec);
 
-    boost::this_thread::sleep(boost::posix_time::microseconds(uSecDelay));
+    boost::this_thread::sleep(boost::posix_time::microseconds(usec_delay));
 
     lasttime.tv_sec = header->ts.tv_sec;
     lasttime.tv_usec = header->ts.tv_usec;
