@@ -56,31 +56,42 @@ bool pcl::io::ply::ply_parser::parse (const std::string& filename)
   format_type format = pcl::io::ply::unknown;
   std::vector< boost::shared_ptr<element> > elements;
 
+  char line_delim = '\n';
+  int char_ignore_count = 0;
+
   // magic
   char magic[4];
   istream.read (magic, 4);
-  if (magic[3] == '\r') // Check if CR/LF
+
+  // Check if CR/LF, setup delim and char skip
+  if (magic[3] == '\r')
+  {
     istream.ignore (1);
+    line_delim = '\r';
+    char_ignore_count = 1;
+  }
+
   ++line_number_;
   if (!istream)
   {
     if (error_callback_)
-      error_callback_ (line_number_, "parse error");
+      error_callback_ (line_number_, "parse error: couldn't read the magic string");
     return false;
   }
 
   if ((magic[0] != 'p') || (magic[1] != 'l') || (magic[2] != 'y'))
   {
     if (error_callback_)
-      error_callback_ (line_number_, "parse error");
+      error_callback_ (line_number_, "parse error: wrong magic string");
     return false;
   }
 
   if (magic_callback_)
     magic_callback_ ();
 
-  while (std::getline (istream, line))
+  while (std::getline (istream, line, line_delim))
   {
+    istream.ignore (char_ignore_count);
     ++line_number_;
     std::istringstream stringstream (line);
     stringstream.unsetf (std::ios_base::skipws);
@@ -101,14 +112,14 @@ bool pcl::io::ply::ply_parser::parse (const std::string& filename)
       {
         std::string format_string, version;
         char space_format_format_string, space_format_string_version;
-        stringstream >> space_format_format_string >> std::ws >> format_string >> space_format_string_version >> std::ws >> version >> std::ws;
+        stringstream >> space_format_format_string >> std::ws >> format_string >> space_format_string_version >> std::ws >> version;
         if (!stringstream || 
             !stringstream.eof () || 
             !isspace (space_format_format_string) || 
             !isspace (space_format_string_version))
         {
           if (error_callback_)
-            error_callback_ (line_number_, "parse error");
+            error_callback_ (line_number_, "parse error: failed to parse the format statement");
           return false;
         }
         if (format_string == "ascii")
@@ -127,7 +138,7 @@ bool pcl::io::ply::ply_parser::parse (const std::string& filename)
         {
           if (error_callback_)
           {
-            error_callback_ (line_number_, "parse error");
+            error_callback_ (line_number_, "parse error: unknown format");
           }
           return false;
         }
@@ -143,7 +154,7 @@ bool pcl::io::ply::ply_parser::parse (const std::string& filename)
         {
           if (error_callback_)
           {
-            error_callback_ (line_number_, "parse error");
+            error_callback_ (line_number_, "parse error: more than 1 format statement");
           }
           return false;
         }
@@ -159,7 +170,7 @@ bool pcl::io::ply::ply_parser::parse (const std::string& filename)
         std::string name;
         std::size_t count;
         char space_element_name, space_name_count;
-        stringstream >> space_element_name >> std::ws >> name >> space_name_count >> std::ws >> count >> std::ws;
+        stringstream >> space_element_name >> std::ws >> name >> space_name_count >> std::ws >> count;
         if (!stringstream || 
             !stringstream.eof () || 
             !isspace (space_element_name) || 
@@ -167,7 +178,7 @@ bool pcl::io::ply::ply_parser::parse (const std::string& filename)
         {
           if (error_callback_)
           {
-            error_callback_ (line_number_, "parse error");
+            error_callback_ (line_number_, "parse error: invalid element statement");
           }
           return false;
         }
@@ -184,7 +195,7 @@ bool pcl::io::ply::ply_parser::parse (const std::string& filename)
         {
           if (error_callback_)
           {
-            error_callback_ (line_number_, "parse error");
+            error_callback_ (line_number_, "parse error: invalid elements");
           }
           return false;
         }
@@ -212,7 +223,7 @@ bool pcl::io::ply::ply_parser::parse (const std::string& filename)
         {
           if (error_callback_)
           {
-            error_callback_ (line_number_, "parse error");
+            error_callback_ (line_number_, "parse error: invalid property statement");
           }
           return false;
         }
@@ -220,7 +231,7 @@ bool pcl::io::ply::ply_parser::parse (const std::string& filename)
           std::string name;
           std::string& type = type_or_list;
           char space_type_name;
-          stringstream >> space_type_name >> std::ws >> name >> std::ws;
+          stringstream >> space_type_name >> std::ws >> name;
           if (!stringstream || !isspace (space_type_name))
           {
             if (error_callback_)
@@ -288,7 +299,7 @@ bool pcl::io::ply::ply_parser::parse (const std::string& filename)
           {
             if (error_callback_)
             {
-              error_callback_ (line_number_, "parse error");
+              error_callback_ (line_number_, "parse error: unknown type");
             }
             return false;
           }
@@ -299,14 +310,14 @@ bool pcl::io::ply::ply_parser::parse (const std::string& filename)
           std::string name;
           std::string size_type_string, scalar_type_string;
           char space_list_size_type, space_size_type_scalar_type, space_scalar_type_name;
-          stringstream >> space_list_size_type >> std::ws >> size_type_string >> space_size_type_scalar_type >> std::ws >> scalar_type_string >> space_scalar_type_name >> std::ws >> name >> std::ws;
+          stringstream >> space_list_size_type >> std::ws >> size_type_string >> space_size_type_scalar_type >> std::ws >> scalar_type_string >> space_scalar_type_name >> std::ws >> name;
           if (!stringstream || 
               !isspace (space_list_size_type) || 
               !isspace (space_size_type_scalar_type) || 
               !isspace (space_scalar_type_name))
           {
             if (error_callback_)
-              error_callback_ (line_number_, "parse error");
+              error_callback_ (line_number_, "parse error: invalid list statement");
             return false;
           }
           if (number_of_element_statements == 0)
@@ -369,7 +380,7 @@ bool pcl::io::ply::ply_parser::parse (const std::string& filename)
             {
               if (error_callback_)
               {
-                error_callback_ (line_number_, "parse error");
+                error_callback_ (line_number_, "parse error: unkonwn scalar type");
               }
               return false;
             }
@@ -412,7 +423,7 @@ bool pcl::io::ply::ply_parser::parse (const std::string& filename)
             else
             {
               if (error_callback_)
-                error_callback_ (line_number_, "parse error");
+                error_callback_ (line_number_, "parse error: unknown scalar type");
               return false;
             }
           }
@@ -454,7 +465,7 @@ bool pcl::io::ply::ply_parser::parse (const std::string& filename)
             else
             {
               if (error_callback_)
-                error_callback_ (line_number_, "parse error");
+                error_callback_ (line_number_, "parse error: unknown scalar type");
               return false;
             }
           }
@@ -506,7 +517,7 @@ bool pcl::io::ply::ply_parser::parse (const std::string& filename)
   if (number_of_format_statements == 0)
   {
     if (error_callback_) 
-     error_callback_ (line_number_, "parse error");
+     error_callback_ (line_number_, "parse error: a format statement is required");
     return false;
   }
 
@@ -522,12 +533,13 @@ bool pcl::io::ply::ply_parser::parse (const std::string& filename)
       {
         if (element.begin_element_callback) 
           element.begin_element_callback ();
-        if (!std::getline (istream, line))
+        if (!std::getline (istream, line, line_delim))
         {
           if (error_callback_)
             error_callback_ (line_number_, "parse error");
           return false;
         }
+        istream.ignore (char_ignore_count);
         ++line_number_;
         std::istringstream stringstream (line);
         stringstream.unsetf (std::ios_base::skipws);
@@ -594,14 +606,19 @@ bool pcl::io::ply::ply_parser::parse (const std::string& filename)
         }
       }
     }
-    if (istream.fail () || (istream.rdbuf ()->sgetc () != std::char_traits<char>::eof ()) || istream.bad ())
+    if (istream.fail () || istream.bad ())
     {
       if (error_callback_)
       {
-        error_callback_ (line_number_, "parse error");
+        error_callback_ (line_number_, "parse error: failed to read from the binary stream");
       }
       return false;
     }
+    if (istream.rdbuf ()->sgetc () != std::char_traits<char>::eof ())
+    {
+      if (warning_callback_)
+        warning_callback_ (line_number_, "ignoring extra data at the end of binary stream");
+    }
     return true;
   }
 }
