@@ -46,6 +46,13 @@
 #include <vtkImageShiftScale.h>
 #include <vtkPNGWriter.h>
 
+// Support for VTK 7.1 upwards
+#ifdef vtkGenericDataArray_h
+#define SetTupleValue SetTypedTuple
+#define InsertNextTupleValue InsertNextTypedTuple
+#define GetTupleValue GetTypedTuple
+#endif
+
 ///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
 int
 pcl::io::loadPolygonFile (const std::string &file_name, pcl::PolygonMesh& mesh)
@@ -74,26 +81,21 @@ pcl::io::loadPolygonFile (const std::string &file_name, pcl::PolygonMesh& mesh)
 }
 
 ///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
-int
-pcl::io::savePolygonFile (const std::string &file_name, const pcl::PolygonMesh& mesh)
+bool
+pcl::io::savePolygonFile (const std::string &file_name,
+                          const pcl::PolygonMesh& mesh,
+                          const bool binary_format)
 {
-  // TODO: what about binary/ASCII modes?!?!?!
   // TODO: what about sensor position and orientation?!?!?!?
-  // TODO: how to adequately catch exceptions thrown by the vtk writers?!
   std::string extension = file_name.substr (file_name.find_last_of (".") + 1);
-  if (extension == "pcd") // no Polygon, but only a point cloud
-  {
-    int error_code = pcl::io::savePCDFile (file_name, mesh.cloud);
-    if (error_code != 0)
-      return (0);
-    return (static_cast<int> (mesh.cloud.width * mesh.cloud.height));
-  }
+  if (extension == "pcd")  // no Polygon, but only a point cloud
+    return (pcl::io::savePCDFile (file_name, mesh.cloud, Eigen::Vector4f::Zero (), Eigen::Quaternionf::Identity (), binary_format) == 0);
   else if (extension == "vtk")
-   return (pcl::io::savePolygonFileVTK (file_name, mesh));
+    return (pcl::io::savePolygonFileVTK (file_name, mesh, binary_format));
   else if (extension == "ply")
-   return (pcl::io::savePolygonFilePLY (file_name, mesh));
-  else if (extension == "stl" )
-    return (pcl::io::savePolygonFileSTL (file_name, mesh));
+    return (pcl::io::savePolygonFilePLY (file_name, mesh, binary_format));
+  else if (extension == "stl")
+    return (pcl::io::savePolygonFileSTL (file_name, mesh, binary_format));
   else
   {
     PCL_ERROR ("[pcl::io::savePolygonFile]: Unsupported file type (%s)\n", extension.c_str ());
@@ -171,8 +173,10 @@ pcl::io::loadPolygonFileSTL (const std::string &file_name, pcl::PolygonMesh& mes
 }
 
 ///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
-int
-pcl::io::savePolygonFileVTK (const std::string &file_name, const pcl::PolygonMesh& mesh)
+bool
+pcl::io::savePolygonFileVTK (const std::string &file_name,
+                             const pcl::PolygonMesh& mesh,
+                             const bool binary_format)
 {
   vtkSmartPointer<vtkPolyData> poly_data = vtkSmartPointer<vtkPolyData>::New ();
 
@@ -184,15 +188,21 @@ pcl::io::savePolygonFileVTK (const std::string &file_name, const pcl::PolygonMes
 #else
   poly_writer->SetInputData (poly_data);
 #endif
-  poly_writer->SetFileName (file_name.c_str ());
-  poly_writer->Write ();
 
-  return (static_cast<int> (mesh.cloud.width * mesh.cloud.height));
+  if (binary_format)
+    poly_writer->SetFileTypeToBinary ();
+  else
+    poly_writer->SetFileTypeToASCII ();
+
+  poly_writer->SetFileName (file_name.c_str ());
+  return (poly_writer->Write ());
 }
 
 ///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
-int
-pcl::io::savePolygonFilePLY (const std::string &file_name, const pcl::PolygonMesh& mesh)
+bool
+pcl::io::savePolygonFilePLY (const std::string &file_name,
+                             const pcl::PolygonMesh& mesh,
+                             const bool binary_format)
 {
   vtkSmartPointer<vtkPolyData> poly_data = vtkSmartPointer<vtkPolyData>::New ();
 
@@ -204,16 +214,22 @@ pcl::io::savePolygonFilePLY (const std::string &file_name, const pcl::PolygonMes
 #else
   poly_writer->SetInputData (poly_data);
 #endif
+
+  if (binary_format)
+    poly_writer->SetFileTypeToBinary ();
+  else
+    poly_writer->SetFileTypeToASCII ();
+
   poly_writer->SetFileName (file_name.c_str ());
 	poly_writer->SetArrayName ("Colors");
-  poly_writer->Write ();
-
-  return (static_cast<int> (mesh.cloud.width * mesh.cloud.height));
+  return (poly_writer->Write ());
 }
 
 ///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
-int
-pcl::io::savePolygonFileSTL (const std::string &file_name, const pcl::PolygonMesh& mesh)
+bool
+pcl::io::savePolygonFileSTL (const std::string &file_name,
+                             const pcl::PolygonMesh& mesh,
+                             const bool binary_format)
 {
   vtkSmartPointer<vtkPolyData> poly_data = vtkSmartPointer<vtkPolyData>::New ();
 
@@ -224,10 +240,14 @@ pcl::io::savePolygonFileSTL (const std::string &file_name, const pcl::PolygonMes
 #else
   poly_writer->SetInputData (poly_data);
 #endif
-  poly_writer->SetFileName (file_name.c_str ());
-  poly_writer->Write ();
 
-  return (static_cast<int> (mesh.cloud.width * mesh.cloud.height));
+  if (binary_format)
+    poly_writer->SetFileTypeToBinary ();
+  else
+    poly_writer->SetFileTypeToASCII ();
+
+  poly_writer->SetFileName (file_name.c_str ());
+  return (poly_writer->Write ());
 }
 
 ///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
@@ -367,7 +387,7 @@ pcl::io::vtk2mesh (const vtkSmartPointer<vtkPolyData>& poly_data, pcl::TextureMe
 
   // Add dummy material
   mesh.tex_materials.push_back (pcl::TexMaterial ());
-  std::vector<Eigen::Vector2f> dummy;
+  std::vector<Eigen::Vector2f, Eigen::aligned_allocator<Eigen::Vector2f> > dummy;
   mesh.tex_coordinates.push_back (dummy);
 
   vtkIdType nr_points = poly_data->GetNumberOfPoints ();
