@@ -358,6 +358,7 @@ namespace Eigen
   template<typename PointT> struct NumTraits<pcl::ndt2d::NormalDist<PointT> >
   {
     typedef double Real;
+    typedef double Literal;
     static Real dummy_precision () { return 1.0; }
     enum {
       IsComplex = 0,
@@ -377,6 +378,9 @@ pcl::NormalDistributionsTransform2D<PointSource, PointTarget>::computeTransforma
 {
   PointCloudSource intm_cloud = output;
 
+  nr_iterations_ = 0;
+  converged_ = false;
+
   if (guess != Eigen::Matrix4f::Identity ())
   {
     transformation_ = guess;
