@@ -48,63 +48,6 @@
 #include <pcl/common/transforms.h>
 
 #include <pcl/features/pfh.h>
-//////////////////////////////////////////////////////////////////////////////////////////////
-void
-pcl::PPFHashMapSearch::setInputFeatureCloud (PointCloud<PPFSignature>::ConstPtr feature_cloud)
-{
-  // Discretize the feature cloud and insert it in the hash map
-  feature_hash_map_->clear ();
-  unsigned int n = static_cast<unsigned int> (sqrt (static_cast<float> (feature_cloud->points.size ())));
-  int d1, d2, d3, d4;
-  max_dist_ = -1.0;
-  alpha_m_.resize (n);
-  for (size_t i = 0; i < n; ++i)
-  {
-    std::vector <float> alpha_m_row (n);
-    for (size_t j = 0; j < n; ++j)
-    {
-      d1 = static_cast<int> (floor (feature_cloud->points[i*n+j].f1 / angle_discretization_step_));
-      d2 = static_cast<int> (floor (feature_cloud->points[i*n+j].f2 / angle_discretization_step_));
-      d3 = static_cast<int> (floor (feature_cloud->points[i*n+j].f3 / angle_discretization_step_));
-      d4 = static_cast<int> (floor (feature_cloud->points[i*n+j].f4 / distance_discretization_step_));
-      feature_hash_map_->insert (std::pair<HashKeyStruct, std::pair<size_t, size_t> > (HashKeyStruct (d1, d2, d3, d4), std::pair<size_t, size_t> (i, j)));
-      alpha_m_row [j] = feature_cloud->points[i*n + j].alpha_m;
-
-      if (max_dist_ < feature_cloud->points[i*n + j].f4)
-        max_dist_ = feature_cloud->points[i*n + j].f4;
-    }
-    alpha_m_[i] = alpha_m_row;
-  }
-
-  internals_initialized_ = true;
-}
-
-
-//////////////////////////////////////////////////////////////////////////////////////////////
-void
-pcl::PPFHashMapSearch::nearestNeighborSearch (float &f1, float &f2, float &f3, float &f4,
-                                              std::vector<std::pair<size_t, size_t> > &indices)
-{
-  if (!internals_initialized_)
-  {
-    PCL_ERROR("[pcl::PPFRegistration::nearestNeighborSearch]: input feature cloud has not been set - skipping search!\n");
-    return;
-  }
-
-  int d1 = static_cast<int> (floor (f1 / angle_discretization_step_)),
-      d2 = static_cast<int> (floor (f2 / angle_discretization_step_)),
-      d3 = static_cast<int> (floor (f3 / angle_discretization_step_)),
-      d4 = static_cast<int> (floor (f4 / distance_discretization_step_));
-
-  indices.clear ();
-  HashKeyStruct key = HashKeyStruct (d1, d2, d3, d4);
-  std::pair <FeatureHashMapType::iterator, FeatureHashMapType::iterator> map_iterator_pair = feature_hash_map_->equal_range (key);
-  for (; map_iterator_pair.first != map_iterator_pair.second; ++ map_iterator_pair.first)
-    indices.push_back (std::pair<size_t, size_t> (map_iterator_pair.first->second.first,
-                                                  map_iterator_pair.first->second.second));
-}
-
-
 //////////////////////////////////////////////////////////////////////////////////////////////
 template <typename PointSource, typename PointTarget> void
 pcl::PPFRegistration<PointSource, PointTarget>::setInputTarget (const PointCloudTargetConstPtr &cloud)
@@ -317,7 +260,7 @@ pcl::PPFRegistration<PointSource, PointTarget>::posesWithinErrorBounds (Eigen::A
                                                                         Eigen::Affine3f &pose2)
 {
   float position_diff = (pose1.translation () - pose2.translation ()).norm ();
-  Eigen::AngleAxisf rotation_diff_mat (pose1.rotation ().inverse () * pose2.rotation ());
+  Eigen::AngleAxisf rotation_diff_mat ((pose1.rotation ().inverse ().lazyProduct (pose2.rotation ()).eval()));
 
   float rotation_diff_angle = fabsf (rotation_diff_mat.angle ());
 
