@@ -57,7 +57,7 @@ namespace pcl
     *   In Proceedings of the 12th International Conference on Intelligent Autonomous Systems (IAS),
     *   Jeju Island, Korea, June 26-29 2012.
     *   <a href="http://purl.org/holz/papers/holz_2012_ias.pdf">http://purl.org/holz/papers/holz_2012_ias.pdf</a>
-    * 
+    *
     * \author Dirk Holz, Radu B. Rusu
     * \ingroup surface
     */
@@ -85,12 +85,20 @@ namespace pcl
 
       /** \brief Constructor. Triangulation type defaults to \a QUAD_MESH. */
       OrganizedFastMesh ()
-      : max_edge_length_squared_ (0.025f)
+      : max_edge_length_a_ (0.0f)
+      , max_edge_length_b_ (0.0f)
+      , max_edge_length_c_ (0.0f)
+      , max_edge_length_set_ (false)
+      , max_edge_length_dist_dependent_ (false)
       , triangle_pixel_size_rows_ (1)
       , triangle_pixel_size_columns_ (1)
       , triangulation_type_ (QUAD_MESH)
+      , viewpoint_ (Eigen::Vector3f::Zero ())
       , store_shadowed_faces_ (false)
       , cos_angle_tolerance_ (fabsf (cosf (pcl::deg2rad (12.5f))))
+      , distance_tolerance_ (-1.0f)
+      , distance_dependent_ (false)
+      , use_depth_as_distance_(false)
       {
         check_tree_ = false;
       };
@@ -98,15 +106,31 @@ namespace pcl
       /** \brief Destructor. */
       virtual ~OrganizedFastMesh () {};
 
-      /** \brief Set a maximum edge length. TODO: Implement!
-        * \param[in] max_edge_length the maximum edge length
+      /** \brief Set a maximum edge length. 
+        * Using not only the scalar \a a, but also \a b and \a c, allows for using a distance threshold in the form of:
+        * threshold(x) = c*x*x + b*x + a 
+        * \param[in] a scalar coefficient of the (distance-dependent polynom) threshold
+        * \param[in] b linear coefficient of the (distance-dependent polynom) threshold
+        * \param[in] c quadratic coefficient of the (distance-dependent polynom) threshold
         */
       inline void
-      setMaxEdgeLength (float max_edge_length)
+      setMaxEdgeLength (float a, float b = 0.0f, float c = 0.0f)
       {
-        max_edge_length_squared_ = max_edge_length * max_edge_length;
+        max_edge_length_a_ = a;
+        max_edge_length_b_ = b;
+        max_edge_length_c_ = c;
+        if ((max_edge_length_a_ + max_edge_length_b_ + max_edge_length_c_) > std::numeric_limits<float>::min())
+          max_edge_length_set_ = true;
+        else
+          max_edge_length_set_ = false;
       };
 
+      inline void
+      unsetMaxEdgeLength ()
+      {
+        max_edge_length_set_  = false;
+      }
+
       /** \brief Set the edge length (in pixels) used for constructing the fixed mesh.
         * \param[in] triangle_size edge length in pixels
         * (Default: 1 = neighboring pixels are connected)
@@ -148,18 +172,74 @@ namespace pcl
         triangulation_type_ = type;
       }
 
+      /** \brief Set the viewpoint from where the input point cloud has been acquired.
+       * \param[in] viewpoint Vector containing the viewpoint coordinates (in the coordinate system of the data)
+       */
+      inline void setViewpoint (const Eigen::Vector3f& viewpoint)
+      {
+        viewpoint_ = viewpoint;
+      }
+
+      /** \brief Get the viewpoint from where the input point cloud has been acquired. */
+      const inline Eigen::Vector3f& getViewpoint () const
+      {
+        return viewpoint_;
+      }
+
       /** \brief Store shadowed faces or not.
-        * \param[in] enable set to true to store shadowed faces
-        */
+       * \param[in] enable set to true to store shadowed faces
+       */
       inline void
       storeShadowedFaces (bool enable)
       {
         store_shadowed_faces_ = enable;
       }
 
+      /** \brief Set the angle tolerance used for checking whether or not an edge is occluded.
+       * Standard values are 5deg to 15deg (input in rad!). Set a value smaller than zero to
+       * disable the check for shadowed edges.
+       * \param[in] angle_tolerance Angle tolerance (in rad). Set a value <0 to disable.
+       */
+      inline void
+      setAngleTolerance(float angle_tolerance)
+      {
+        if (angle_tolerance > 0)
+          cos_angle_tolerance_ = fabsf (cosf (angle_tolerance));
+        else
+          cos_angle_tolerance_ = -1.0f;
+      }
+
+
+      inline void setDistanceTolerance(float distance_tolerance, bool depth_dependent = false)
+      {
+        distance_tolerance_ = distance_tolerance;
+        if (distance_tolerance_ < 0)
+          return;
+
+        distance_dependent_ = depth_dependent;
+        if (!distance_dependent_)
+          distance_tolerance_ *= distance_tolerance_;
+      }
+
+      /** \brief Use the points' depths (z-coordinates) instead of measured distances (points' distances to the viewpoint).
+        * \param[in] enable Set to true skips comptations and further speeds up computation by using depth instead of computing distance. false to disable. */
+      inline void useDepthAsDistance(bool enable)
+      {
+        use_depth_as_distance_ = enable;
+      }
+
     protected:
-      /** \brief max (squared) length of edge */
-      float max_edge_length_squared_;
+      /** \brief max length of edge, scalar component */
+      float max_edge_length_a_;
+      /** \brief max length of edge, scalar component */
+      float max_edge_length_b_;
+      /** \brief max length of edge, scalar component */
+      float max_edge_length_c_;
+      /** \brief flag whether or not edges are limited in length */
+      bool max_edge_length_set_;
+
+      /** \brief flag whether or not max edge length is distance dependent. */
+      bool max_edge_length_dist_dependent_;
 
       /** \brief size of triangle edges (in pixels) for iterating over rows. */
       int triangle_pixel_size_rows_;
@@ -170,12 +250,26 @@ namespace pcl
       /** \brief Type of meshing scheme (quads vs. triangles, left cut vs. right cut ... */
       TriangulationType triangulation_type_;
 
+      /** \brief Viewpoint from which the point cloud has been acquired (in the same coordinate frame as the data). */
+      Eigen::Vector3f viewpoint_;
+
       /** \brief Whether or not shadowed faces are stored, e.g., for exploration */
       bool store_shadowed_faces_;
 
       /** \brief (Cosine of the) angle tolerance used when checking whether or not an edge between two points is shadowed. */
       float cos_angle_tolerance_;
 
+      /** \brief distance tolerance for filtering out shadowed/occluded edges */
+      float distance_tolerance_;
+
+      /** \brief flag whether or not \a distance_tolerance_ is distance dependent (multiplied by the squared distance to the point) or not. */
+      bool distance_dependent_;
+
+      /** \brief flag whether or not the points' depths are used instead of measured distances (points' distances to the viewpoint).
+          This flag may be set using useDepthAsDistance(true) for (RGB-)Depth cameras to skip computations and gain additional speed up. */
+      bool use_depth_as_distance_;
+
+
       /** \brief Perform the actual polygonal reconstruction.
         * \param[out] polygons the resultant polygons
         */
@@ -259,16 +353,50 @@ namespace pcl
       inline bool
       isShadowed (const PointInT& point_a, const PointInT& point_b)
       {
-        Eigen::Vector3f viewpoint = Eigen::Vector3f::Zero (); // TODO: allow for passing viewpoint information
-        Eigen::Vector3f dir_a = viewpoint - point_a.getVector3fMap ();
+        bool valid = true;
+
+        Eigen::Vector3f dir_a = viewpoint_ - point_a.getVector3fMap ();
         Eigen::Vector3f dir_b = point_b.getVector3fMap () - point_a.getVector3fMap ();
         float distance_to_points = dir_a.norm ();
         float distance_between_points = dir_b.norm ();
-        float cos_angle = dir_a.dot (dir_b) / (distance_to_points*distance_between_points);
-        if (cos_angle != cos_angle)
-          cos_angle = 1.0f;
-        return (fabs (cos_angle) >= cos_angle_tolerance_);
-        // TODO: check for both: angle almost 0/180 _and_ distance between points larger than noise level
+
+        if (cos_angle_tolerance_ > 0)
+        {
+          float cos_angle = dir_a.dot (dir_b) / (distance_to_points*distance_between_points);
+          if (cos_angle != cos_angle)
+            cos_angle = 1.0f;
+          bool check_angle = fabs (cos_angle) >= cos_angle_tolerance_;
+
+          bool check_distance = true;
+          if (check_angle && (distance_tolerance_ > 0))
+          {
+            float dist_thresh = distance_tolerance_;
+            if (distance_dependent_)
+            {
+              float d = distance_to_points;
+              if (use_depth_as_distance_)
+                d = std::max(point_a.z, point_b.z);
+              dist_thresh *= d*d;
+              dist_thresh *= dist_thresh;  // distance_tolerance_ is already squared if distance_dependent_ is false.
+            }
+            check_distance = (distance_between_points > dist_thresh);
+          }
+          valid = !(check_angle && check_distance);
+        }
+
+        // check if max. edge length is not exceeded
+        if (max_edge_length_set_)
+        {
+          float dist = (use_depth_as_distance_ ? std::max(point_a.z, point_b.z) : distance_to_points);
+          float dist_thresh = max_edge_length_a_;
+          if (fabs(max_edge_length_b_) > std::numeric_limits<float>::min())
+            dist_thresh += max_edge_length_b_ * dist;
+          if (fabs(max_edge_length_c_) > std::numeric_limits<float>::min())
+            dist_thresh += max_edge_length_c_ * dist * dist;
+          valid = (distance_between_points <= dist_thresh);
+        }
+
+        return !valid;
       }
 
       /** \brief Check if a triangle is valid.
