@@ -36,6 +36,7 @@
  */
 
 #include <pcl/surface/on_nurbs/fitting_curve_2d_apdm.h>
+#include <pcl/pcl_macros.h>
 #include <stdexcept>
 
 using namespace pcl;
@@ -116,8 +117,6 @@ FittingCurve2dAPDM::refine (double xi)
 void
 FittingCurve2dAPDM::fitting (FitParameter &param)
 {
-  double avgerr (DBL_MAX);
-  double maxerr (DBL_MAX);
   bool stop (false);
   for (unsigned j = 0; j < param.iterations && !stop; j++)
   {
