@@ -203,6 +203,7 @@ TEST (PCL, IterativeClosestPoint)
 void
 sampleRandomTransform (Eigen::Affine3f &trans, float max_angle, float max_trans)
 {
+    srand(0);
     // Sample random transform
     Eigen::Vector3f axis((float)rand() / RAND_MAX, (float)rand() / RAND_MAX, (float)rand() / RAND_MAX);
     axis.normalize();
@@ -235,19 +236,19 @@ TEST (PCL, IterativeClosestPointWithRejectors)
     Eigen::Affine3f delta_transform;
     sampleRandomTransform (delta_transform, 0., 0.05);
     // Sample random global transform for each pair, to make sure we aren't biased around the origin
-    Eigen::Affine3f net_transform;    
+    Eigen::Affine3f net_transform;
     sampleRandomTransform (net_transform, 2*M_PI, 10.);
-      
+
     PointCloud<PointXYZ>::ConstPtr source (cloud_source.makeShared ());
     PointCloud<PointXYZ>::Ptr source_trans (new PointCloud<PointXYZ>);
     PointCloud<PointXYZ>::Ptr target_trans (new PointCloud<PointXYZ>);
-      
+
     pcl::transformPointCloud (*source, *source_trans, delta_transform.inverse () * net_transform);
     pcl::transformPointCloud (*source, *target_trans, net_transform);
 
     reg.setInputSource (source_trans);
     reg.setInputTarget (target_trans);
-    
+
     // Register
     reg.align (cloud_reg);
     Eigen::Matrix4f trans_final = reg.getFinalTransformation ();
@@ -281,7 +282,7 @@ TEST (PCL, JointIterativeClosestPoint)
   size_t ntransforms = 10;
   for (size_t t = 0; t < ntransforms; t++)
   {
-    
+
     // Sample a fixed offset between cloud pairs
     Eigen::Affine3f delta_transform;
     // No rotation, since at a random offset this could make it converge to a wrong (but still reasonable) result
@@ -365,7 +366,7 @@ TEST (PCL, IterativeClosestPointNonLinear)
   EXPECT_EQ (transformation (3, 3), 1);
   */
   EXPECT_LT (reg.getFitnessScore (), 0.001);
- 
+
   // Check again, for all possible caching schemes
   for (int iter = 0; iter < 4; iter++)
   {
@@ -381,7 +382,7 @@ TEST (PCL, IterativeClosestPointNonLinear)
     if (force_cache_reciprocal)
       tree_recip->setInputCloud (temp_src);
     reg.setSearchMethodSource (tree_recip, force_cache_reciprocal);
-    
+
     // Register
     reg.align (output);
     EXPECT_EQ (int (output.points.size ()), int (cloud_source.points.size ()));
@@ -442,13 +443,13 @@ TEST (PCL, IterativeClosestPoint_PointToPlane)
     if (force_cache_reciprocal)
       tree_recip->setInputCloud (src);
     reg.setSearchMethodSource (tree_recip, force_cache_reciprocal);
-    
+
     // Register
     reg.align (output);
     EXPECT_EQ (int (output.points.size ()), int (cloud_source.points.size ()));
     EXPECT_LT (reg.getFitnessScore (), 0.005);
   }
-  
+
 
 
   // We get different results on 32 vs 64-bit systems.  To address this, we've removed the explicit output test
@@ -489,7 +490,6 @@ TEST (PCL, GeneralizedIterativeClosestPoint)
   copyPointCloud (cloud_target, *tgt);
   PointCloud<PointT> output;
 
-
   GeneralizedIterativeClosestPoint<PointT, PointT> reg;
   reg.setInputSource (src);
   reg.setInputTarget (tgt);
@@ -499,7 +499,7 @@ TEST (PCL, GeneralizedIterativeClosestPoint)
   // Register
   reg.align (output);
   EXPECT_EQ (int (output.points.size ()), int (cloud_source.points.size ()));
-  EXPECT_LT (reg.getFitnessScore (), 0.001);
+  EXPECT_LT (reg.getFitnessScore (), 0.0001);
 
   // Check again, for all possible caching schemes
   for (int iter = 0; iter < 4; iter++)
@@ -516,12 +516,29 @@ TEST (PCL, GeneralizedIterativeClosestPoint)
     if (force_cache_reciprocal)
       tree_recip->setInputCloud (src);
     reg.setSearchMethodSource (tree_recip, force_cache_reciprocal);
-    
+
     // Register
     reg.align (output);
     EXPECT_EQ (int (output.points.size ()), int (cloud_source.points.size ()));
     EXPECT_LT (reg.getFitnessScore (), 0.001);
   }
+
+  // Test guess matrix
+  Eigen::Isometry3f transform = Eigen::Isometry3f (Eigen::AngleAxisf (0.25 * M_PI, Eigen::Vector3f::UnitX ())
+                                                 * Eigen::AngleAxisf (0.50 * M_PI, Eigen::Vector3f::UnitY ())
+                                                 * Eigen::AngleAxisf (0.33 * M_PI, Eigen::Vector3f::UnitZ ()));
+  transform.translation () = Eigen::Vector3f (0.1, 0.2, 0.3);
+  PointCloud<PointT>::Ptr transformed_tgt (new PointCloud<PointT>);
+  pcl::transformPointCloud (*tgt, *transformed_tgt, transform.matrix ()); // transformed_tgt is now a copy of tgt with a transformation matrix applied
+
+  GeneralizedIterativeClosestPoint<PointT, PointT> reg_guess;
+  reg_guess.setInputSource (src);
+  reg_guess.setInputTarget (transformed_tgt);
+  reg_guess.setMaximumIterations (50);
+  reg_guess.setTransformationEpsilon (1e-8);
+  reg_guess.align (output, transform.matrix ());
+  EXPECT_EQ (int (output.points.size ()), int (cloud_source.points.size ()));
+  EXPECT_LT (reg.getFitnessScore (), 0.0001);
 }
 
 //////////////////////////////////////////////////////////////////////////////////////////////////////////////////
@@ -601,7 +618,7 @@ TEST (PCL, NormalDistributionsTransform)
   reg.align (output);
   EXPECT_EQ (int (output.points.size ()), int (cloud_source.points.size ()));
   EXPECT_LT (reg.getFitnessScore (), 0.001);
-  
+
   // Check again, for all possible caching schemes
   for (int iter = 0; iter < 4; iter++)
   {
@@ -617,7 +634,7 @@ TEST (PCL, NormalDistributionsTransform)
     if (force_cache_reciprocal)
       tree_recip->setInputCloud (src);
     reg.setSearchMethodSource (tree_recip, force_cache_reciprocal);
-    
+
     // Register
     reg.align (output);
     EXPECT_EQ (int (output.points.size ()), int (cloud_source.points.size ()));
@@ -683,7 +700,7 @@ TEST (PCL, SampleConsensusInitialAlignment)
   reg.align (cloud_reg);
   EXPECT_EQ (int (cloud_reg.points.size ()), int (cloud_source.points.size ()));
   EXPECT_LT (reg.getFitnessScore (), 0.0005);
-  
+
   // Check again, for all possible caching schemes
   typedef pcl::PointXYZ PointT;
   for (int iter = 0; iter < 4; iter++)
@@ -700,7 +717,7 @@ TEST (PCL, SampleConsensusInitialAlignment)
     if (force_cache_reciprocal)
       tree_recip->setInputCloud (cloud_source_ptr);
     reg.setSearchMethodSource(tree_recip, force_cache_reciprocal);
-    
+
     // Register
     reg.align (cloud_reg);
     EXPECT_EQ (int (cloud_reg.points.size ()), int (cloud_source.points.size ()));
@@ -718,7 +735,7 @@ TEST (PCL, SampleConsensusPrerejective)
    *   1) the number of iterations is increased 1000 --> 5000
    *   2) the feature correspondence randomness (the number of kNNs) is decreased 10 --> 2
    */
-  
+
   // Transform the source cloud by a large amount
   Eigen::Vector3f initial_offset (100, 0, 0);
   float angle = static_cast<float> (M_PI) / 2.0f;
@@ -760,13 +777,13 @@ TEST (PCL, SampleConsensusPrerejective)
   fpfh_est.setInputNormals (normals.makeShared ());
   fpfh_est.compute (features_target);
 
-  // Initialize Sample Consensus Prerejective with 5x the number of iterations and 1/5 feature kNNs as SAC-IA 
+  // Initialize Sample Consensus Prerejective with 5x the number of iterations and 1/5 feature kNNs as SAC-IA
   SampleConsensusPrerejective<PointXYZ, PointXYZ, FPFHSignature33> reg;
   reg.setMaxCorrespondenceDistance (0.1);
   reg.setMaximumIterations (5000);
   reg.setSimilarityThreshold (0.6f);
   reg.setCorrespondenceRandomness (2);
-  
+
   // Set source and target cloud/features
   reg.setInputSource (cloud_source_ptr);
   reg.setInputTarget (cloud_target_ptr);
@@ -775,12 +792,12 @@ TEST (PCL, SampleConsensusPrerejective)
 
   // Register
   reg.align (cloud_reg);
-  
+
   // Check output consistency and quality of alignment
   EXPECT_EQ (static_cast<int> (cloud_reg.points.size ()), static_cast<int> (cloud_source.points.size ()));
   float inlier_fraction = static_cast<float> (reg.getInliers ().size ()) / static_cast<float> (cloud_source.points.size ());
   EXPECT_GT (inlier_fraction, 0.95f);
-  
+
   // Check again, for all possible caching schemes
   typedef pcl::PointXYZ PointT;
   for (int iter = 0; iter < 4; iter++)
@@ -797,7 +814,7 @@ TEST (PCL, SampleConsensusPrerejective)
     if (force_cache_reciprocal)
       tree_recip->setInputCloud (cloud_source_ptr);
     reg.setSearchMethodSource(tree_recip, force_cache_reciprocal);
-    
+
     // Register
     reg.align (cloud_reg);
 
