@@ -226,15 +226,13 @@ pcl::tracking::PyramidalKLTTracker<PointInT, IntensityT>::downsample (const Floa
 
   int width = (smoothed.width +1) / 2;
   int height = (smoothed.height +1) / 2;
-
-  int *ii = new int[width];
-  int *ii_ptr = ii;
+  std::vector<int> ii (width);
   for (int i = 0; i < width; ++i)
-    *ii_ptr++ = 2*i;
+    ii[i] = 2 * i;
 
   FloatImagePtr down (new FloatImage (width, height));
 #ifdef _OPENMP
-#pragma omp parallel for shared (output) private (ii) num_threads (threads_)
+#pragma omp parallel for shared (output) firstprivate (ii) num_threads (threads_)
 #endif
   for (int j = 0; j < height; ++j)
   {
@@ -482,7 +480,7 @@ pcl::tracking::PyramidalKLTTracker<PointInT, IntensityT>::track (const PointClou
                                                                  std::vector<int>& status,
                                                                  Eigen::Affine3f& motion) const
 {
-  std::vector<Eigen::Array2f> next_pts (prev_keypoints->size ());
+  std::vector<Eigen::Array2f, Eigen::aligned_allocator<Eigen::Array2f> > next_pts (prev_keypoints->size ());
   Eigen::Array2f half_win ((track_width_-1)*0.5f, (track_height_-1)*0.5f);
   pcl::TransformationFromCorrespondences transformation_computer;
   const int nb_points = prev_keypoints->size ();
