@@ -126,10 +126,17 @@ pcl::visualization::ImageViewer::ImageViewer (const std::string& window_title)
 #if VTK_MAJOR_VERSION < 6
   map->SetInput (empty_image);
 #else
+ #if ((VTK_MAJOR_VERSION == 6) && (VTK_MINOR_VERSION == 1))
+  empty_image->AllocateScalars (VTK_UNSIGNED_CHAR, 3);
+  algo_->SetInputData (empty_image);
+  map->SetInputConnection (algo_->GetOutputPort ());
+ #else
   map->SetInputData (empty_image);
+ #endif
 #endif
   slice_->SetMapper (map);
   ren_->AddViewProp (slice_);
+  ren_->GetActiveCamera ()->ParallelProjectionOn ();
   interactor_->SetInteractorStyle (interactor_style_);
 #endif
 
@@ -213,6 +220,9 @@ pcl::visualization::ImageViewer::addRGBImage (
 #else
   algo_->SetInputData (image);
   algo_->Update ();
+  slice_->GetMapper ()->SetInputConnection (algo_->GetOutputPort ());
+  ren_->ResetCamera ();
+  ren_->GetActiveCamera ()->SetParallelScale (0.5 * win_->GetSize ()[1]);
 #endif
 }
 
@@ -272,6 +282,9 @@ pcl::visualization::ImageViewer::addMonoImage (
 #else
   algo_->SetInputData (image);
   algo_->Update ();
+  slice_->GetMapper ()->SetInputConnection (algo_->GetOutputPort ());
+  ren_->ResetCamera ();
+  ren_->GetActiveCamera ()->SetParallelScale (0.5 * win_->GetSize ()[1]);
 #endif
 }
 
@@ -692,7 +705,7 @@ pcl::visualization::ImageViewer::addCircle (
                      static_cast<unsigned char> (255.0 * g),
                      static_cast<unsigned char> (255.0 * b));
   circle->setOpacity (opacity);
-#if ((VTK_MAJOR_VERSION == 5) && (VTK_MINOR_VERSION > 10))
+#if ((VTK_MAJOR_VERSION >= 6) || ((VTK_MAJOR_VERSION == 5) && (VTK_MINOR_VERSION > 7)))
   circle->set (static_cast<float> (x), static_cast<float> (y), static_cast<float> (radius));
 #else
   circle->set (static_cast<float> (x), static_cast<float> (getSize ()[1] - y), static_cast<float> (radius));
@@ -732,7 +745,7 @@ pcl::visualization::ImageViewer::addFilledRectangle (
                    static_cast<unsigned char> (255.0 * g),
                    static_cast<unsigned char> (255.0 * b));
   rect->setOpacity (opacity);
-#if ((VTK_MAJOR_VERSION == 5) && (VTK_MINOR_VERSION > 10))
+#if ((VTK_MAJOR_VERSION >= 6) || (VTK_MAJOR_VERSION == 5) && (VTK_MINOR_VERSION > 7))
   rect->set (static_cast<float> (x_min), static_cast<float> (y_min),
              static_cast<float> (x_max - x_min), static_cast<float> (y_max - y_min));
 #else
@@ -775,7 +788,7 @@ pcl::visualization::ImageViewer::addRectangle (
                    static_cast<unsigned char> (255.0 * g),
                    static_cast<unsigned char> (255.0 * b));
   rect->setOpacity (opacity);
-#if ((VTK_MAJOR_VERSION == 5) && (VTK_MINOR_VERSION > 7))
+#if ((VTK_MAJOR_VERSION >= 6) || ((VTK_MAJOR_VERSION == 5) && (VTK_MINOR_VERSION > 7)))
   rect->set (static_cast<float> (x_min), static_cast<float> (y_min),
              static_cast<float> (x_max), static_cast<float> (y_max));
 #else
@@ -818,7 +831,7 @@ pcl::visualization::ImageViewer::addRectangle (
                    static_cast<unsigned char> (255.0 * g),
                    static_cast<unsigned char> (255.0 * b));
   rect->setOpacity (opacity);
-#if ((VTK_MAJOR_VERSION == 5) && (VTK_MINOR_VERSION > 7))
+#if ((VTK_MAJOR_VERSION >= 6) ||((VTK_MAJOR_VERSION == 5) && (VTK_MINOR_VERSION > 7)))
   rect->set (min_pt.x, min_pt.y, max_pt.x, max_pt.y);
 #else
   rect->set (min_pt.x, static_cast<float> (getSize ()[1]) - min_pt.y, max_pt.x, max_pt.y);
@@ -860,7 +873,7 @@ pcl::visualization::ImageViewer::addLine (unsigned int x_min, unsigned int y_min
                    static_cast<unsigned char> (255.0 * g),
                    static_cast<unsigned char> (255.0 * b));
   line->setOpacity (opacity);
-#if ((VTK_MAJOR_VERSION == 5) && (VTK_MINOR_VERSION > 10))
+#if ((VTK_MAJOR_VERSION >= 6) || ((VTK_MAJOR_VERSION == 5) && (VTK_MINOR_VERSION > 7)))
   line->set (static_cast<float> (x_min), static_cast<float> (y_min),
              static_cast<float> (x_max), static_cast<float> (y_max));
 #else
@@ -904,7 +917,7 @@ pcl::visualization::ImageViewer::addText (unsigned int x, unsigned int y,
                    static_cast<unsigned char> (255.0 * g),
                    static_cast<unsigned char> (255.0 * b));
   text->setOpacity (opacity);
-#if ((VTK_MAJOR_VERSION == 5) && (VTKOR_VERSION > 10))
+#if ((VTK_MAJOR_VERSION >= 6) || ((VTK_MAJOR_VERSION == 5) && (VTK_MINOR_VERSION > 7)))
   text->set (static_cast<float> (x), static_cast<float> (y), text_string);
 #else
   text->set (static_cast<float> (x), static_cast<float> (getSize ()[1] - y), text_string);
@@ -947,7 +960,7 @@ pcl::visualization::ImageViewer::markPoint (
   disk->setColors (bg_color[0], bg_color[1], bg_color[2]);
   disk->setOpacity (opacity);
 
-#if ((VTK_MAJOR_VERSION == 5) && (VTK_MINOR_VERSION > 10))
+#if ((VTK_MAJOR_VERSION >= 6) || ((VTK_MAJOR_VERSION == 5) && (VTK_MINOR_VERSION > 7)))
   point->set (static_cast<float> (u), static_cast<float> (v));
   disk->set (static_cast<float> (u), static_cast<float> (v), static_cast<float> (radius));
 #else
@@ -996,7 +1009,7 @@ pcl::visualization::ImageViewer::markPoints (
 
   vtkSmartPointer<context_items::Markers> markers = vtkSmartPointer<context_items::Markers>::New ();
   markers->setOpacity (opacity);
-#if ((VTK_MAJOR_VERSION == 5) && (VTK_MINOR_VERSION > 10))
+#if ((VTK_MAJOR_VERSION >= 6) || ((VTK_MAJOR_VERSION == 5) && (VTK_MINOR_VERSION > 7)))
   markers->set (uv);
 #else
   // translate v which is on odd indices
